"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const grpc_boom_1 = __importDefault(require("grpc-boom"));
const health_grpc_pb_1 = require("./proto/health_grpc_pb");
exports.HealthClient = health_grpc_pb_1.HealthClient;
exports.HealthService = health_grpc_pb_1.HealthService;
const health_pb_1 = require("./proto/health_pb");
exports.HealthCheckRequest = health_pb_1.HealthCheckRequest;
exports.HealthCheckResponse = health_pb_1.HealthCheckResponse;
class GrpcHealthCheck {
    constructor(statusMap) {
        this.statusMap = statusMap;
        this.watchStatusMap = {};
        this.watchErrorMap = {};
    }
    setStatus(service, status) {
        this.statusMap[service] = status;
    }
    sendStatusResponse(call, status, callback) {
        const response = new health_pb_1.HealthCheckResponse();
        response.setStatus(status);
        call.write(response, callback);
    }
    check(call, callback) {
        const service = call.request.getService();
        const status = this.statusMap[service];
        if (!status) {
            callback(grpc_boom_1.default.notFound(`Unknown service: ${service}`), null);
        }
        else {
            const response = new health_pb_1.HealthCheckResponse();
            response.setStatus(status);
            callback(null, response);
        }
    }
    watch(call) {
        const service = call.request.getService();
        const interval = setInterval(() => {
            let updatedStatus = health_pb_1.HealthCheckResponse.ServingStatus.SERVING;
            if (!this.statusMap[service]) {
                updatedStatus = health_pb_1.HealthCheckResponse.ServingStatus.SERVICE_UNKNOWN;
                this.setStatus(service, updatedStatus);
                this.sendStatusResponse(call, updatedStatus);
            }
            this.watchStatusMap[service] = updatedStatus;
            if (!this.watchErrorMap[service]) {
                const lastStatus = this.statusMap[service] || -1;
                if (lastStatus !== updatedStatus) {
                    this.setStatus(service, updatedStatus);
                    this.sendStatusResponse(call, updatedStatus, (error) => {
                        if (error) {
                            this.watchErrorMap[service] = error;
                        }
                    });
                }
            }
            else {
                clearInterval(interval);
                call.end(this.watchErrorMap[service]);
            }
        }, 1000);
    }
}
exports.GrpcHealthCheck = GrpcHealthCheck;
